# Mattermost Logger

A Drupal module that integrates with Mattermost via incoming webhooks
to log messages from the Drupal logging system.
Choose which logging channels to send to Mattermost,
and set the logging levels for each channel.

## Features

- Hooks into Drupal's logger service.
- Supports multiple logging channels.
- Configurable logging levels per channel.
- Allows setting a unique webhook per channel or default to a default webhook.
- Displays messages with color-coded indicators in Mattermost.
- Supports advanced Mattermost message attachments via optional parameters.


## Requirements

- A Mattermost server with an incoming webhook.
- This module requires no modules outside of Drupal core.

## Installation

- Install the module using Composer:
  ```bash
  composer require drupal/mattermost_logger
  ```
  or manually clone this module to the custom modules directory.
- Enable the module:
  ```bash
  drush en mattermost_logger
  ```


## Configuration

- Navigate to `/admin/config/services/mattermost-logger/settings`.
- Add drupal logging channels and their corresponding webhooks.
- Select the logging levels for each channel.

## Usage

- The module automatically logs messages based on the configured
  channels and levels.
- Developers can use the `MattermostLogger` service manually:
  - For example:
    ```php
    \Drupal::service('mattermost_logger')
      ->error('my_module', 'My error message');
    ```

### Advanced Usage: Custom Attachments
- The `sendMessage()` function supports a fifth optional `$optional` parameter
that allows sending rich message [attachments](https://developers.mattermost.com/integrate/reference/message-attachments/) to Mattermost. 
  - For example:
    ```php
    \Drupal::service('mattermost_logger')->sendMessage(
      'channel_name',
      'Some message to send',
      'INFO',
      optional: [
        "title" => "This is a title!",
      ]
    );
      ```