<?php

namespace Drupal\mattermost_logger\Logger;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\mattermost_logger\MattermostLogger;
use Psr\Log\LoggerInterface;
use Psr\Log\LogLevel;

/**
 * Logs messages to Mattermost based on the channel and log level.
 */
class MattermostAutoLogger implements LoggerInterface {
  use RfcLoggerTrait;

  /**
   * Channel webhooks.
   *
   * @var array|null
   *   Channel webhooks.
   */
  private ?array $channelWebhooks;

  /**
   * Map of RFC 5424 log levels to PSR-3 log levels.
   *
   * @var array
   */
  const LEVEL_TRANSLATION = [
    RfcLogLevel::EMERGENCY => LogLevel::EMERGENCY,
    RfcLogLevel::ALERT => LogLevel::ALERT,
    RfcLogLevel::CRITICAL => LogLevel::CRITICAL,
    RfcLogLevel::ERROR => LogLevel::ERROR,
    RfcLogLevel::WARNING => LogLevel::WARNING,
    RfcLogLevel::NOTICE => LogLevel::NOTICE,
    RfcLogLevel::INFO => LogLevel::INFO,
    RfcLogLevel::DEBUG => LogLevel::DEBUG,
  ];

  public function __construct(protected MattermostLogger $mattermostLogger, protected ConfigFactoryInterface $configFactory) {
    $this->channelWebhooks = $this->configFactory->get('mattermost_logger.settings')->get('channel_webhooks');
  }

  /**
   * {@inheritDoc}
   */
  public function log($level, \Stringable|string $message, array $context = []): void {
    $current_level = self::LEVEL_TRANSLATION[$level];
    $channel = $context['channel'] ?? 'none';

    $webhook_channel = array_filter($this->channelWebhooks ?? [], fn($webhook) => $webhook['channel'] === $channel);
    $webhook_channel = reset($webhook_channel);

    if (empty($webhook_channel)) {
      return;
    }

    $webhook_level = array_filter($webhook_channel['level'] ?? [], fn($webhook_level) => $webhook_level === $current_level);
    $webhook_level = reset($webhook_level);

    if (empty($webhook_level)) {
      return;
    }

    if (empty($webhook_channel['use_default_webhook_url'])) {
      $this->mattermostLogger->setWebhookUrl($webhook_channel['webhook_url']);
    }

    $this->mattermostLogger->sendMessage($channel, $message, $current_level, $context);
  }

}
